cimport numpy as np
np.import_array()

from libc.stdlib cimport malloc, free

# cdefine the signature of our c function
cdef extern from "dbscan.h":
    int dbscan(long **all_points,
               long **dist_matrix,
               long *cluster_assignments,
               int num_points,
               int num_dimensions,
               int alphabet_size,
               int epsilon_cutoff,
               int min_neighbors,
               int dist_function)

# define our simple wrap function
def run_dbscan(np.ndarray[long, ndim=2, mode="c"] all_points not None, 
                np.ndarray[long, ndim=2, mode="c"] dist_matrix not None,
                np.ndarray[long, ndim=1, mode="c"] cluster_assignments not None,
                num_points,
                num_dimensions,
                alphabet_size,
                epsilon, 
                min_neighbors,
                dist_function):

    cdef:
        long **_all_points
        long **_dist_matrix

    _all_points = <long **> malloc(sizeof(long *) * num_points)
    _dist_matrix = <long **> malloc(sizeof(long *) * alphabet_size)
    
    for i in range(num_points):
        _all_points[i] = <long *> np.PyArray_DATA(all_points[i,:])

    for i in range(alphabet_size):
        _dist_matrix[i] = <long *> np.PyArray_DATA(dist_matrix[i,:])

    status = dbscan(_all_points,
                    _dist_matrix,
                    <long*> np.PyArray_DATA(cluster_assignments),
                    <int> num_points,
                    <int> num_dimensions,
                    <int> alphabet_size,
                    <int> epsilon,
                    <int> min_neighbors,
                    <int> dist_function)

    free(_all_points)
    free(_dist_matrix)

    return status



import sys 
if sys.version_info[0] < 3:
    sys.exit('Sorry, Python < 3.x is not supported')

from setuptools import setup, find_packages, Extension
import numpy
from Cython.Distutils import build_ext

setup(name="fast_dbscan",
      packages=find_packages(),
      cmdclass={'build_ext': build_ext},
      version='0.1.0',
      description="fast dbscan clustering on peptide strings",
      #long_description=open("README.md").read(),
      author='Michael J. Harms',
      author_email='harmsm@gmail.com',
      url='https://github.com/harmslab/fast_dbscan',
      download_url='https://github.com/harmslab/fast_dbscan/archive/0.1.0.tar.gz',
      ext_modules=[Extension("dbscan",
                   sources=["src/dbscan_module.pyx", "src/dbscan.c"],
                   include_dirs=["src/",numpy.get_include()])],
      install_requires=["numpy"],
      zip_safe=False,
      package_data={"":["*.h","src/*.h"]},
      classifiers=['Programming Language :: Python'],
      entry_points = {
            'console_scripts': [
                'fast_dbscan = fast_dbscan.fast_dbscan:main',
            ]
      })
